@@ -35,165 +35,6 @@
 #define BLOCKS_PER_UNIT		(1U << UNIT_SHIFT)
 #define BLOCKS_PER_UNIT_MASK	(BLOCKS_PER_UNIT - 1)
 
-#define firstfree_valid(b)	((b)->first_free != (uint64_t) -1)
-
-struct axmap_level {
-	int level;
-	unsigned long map_size;
-	unsigned long *map;
-};
-
-struct axmap {
-	unsigned int nr_levels;
-	struct axmap_level *levels;
-	uint64_t first_free;
-	uint64_t nr_bits;
-};
-
-static unsigned long ulog64(unsigned long val, unsigned int log)
-{
-	while (log-- && val)
-		val >>= UNIT_SHIFT;
-
-	return val;
-}
-
-void axmap_reset(struct axmap *axmap)
-{
-	int i;
-
-	for (i = 0; i < axmap->nr_levels; i++) {
-		struct axmap_level *al = &axmap->levels[i];
-
-		memset(al->map, 0, al->map_size * sizeof(unsigned long));
-	}
-
-	axmap->first_free = 0;
-}
-
-void axmap_free(struct axmap *axmap)
-{
-	unsigned int i;
-
-	if (!axmap)
-		return;
-
-	for (i = 0; i < axmap->nr_levels; i++)
-		free(axmap->levels[i].map);
-
-	free(axmap->levels);
-	free(axmap);
-}
-
-struct axmap *axmap_new(unsigned long nr_bits)
-{
-	struct axmap *axmap;
-	unsigned int i, levels;
-
-	axmap = malloc(sizeof(*axmap));
-	if (!axmap)
-		return NULL;
-
-	levels = 1;
-	i = (nr_bits + BLOCKS_PER_UNIT - 1) >> UNIT_SHIFT;
-	while (i > 1) {
-		i = (i + BLOCKS_PER_UNIT - 1) >> UNIT_SHIFT;
-		levels++;
-	}
-
-	axmap->nr_levels = levels;
-	axmap->levels = calloc(axmap->nr_levels, sizeof(struct axmap_level));
-	axmap->nr_bits = nr_bits;
-
-	for (i = 0; i < axmap->nr_levels; i++) {
-		struct axmap_level *al = &axmap->levels[i];
-
-		al->level = i;
-		al->map_size = (nr_bits + BLOCKS_PER_UNIT - 1) >> UNIT_SHIFT;
-		al->map = malloc(al->map_size * sizeof(unsigned long));
-		if (!al->map)
-			goto err;
-
-		nr_bits = (nr_bits + BLOCKS_PER_UNIT - 1) >> UNIT_SHIFT;
-	}
-
-	axmap_reset(axmap);
-	return axmap;
-err:
-	for (i = 0; i < axmap->nr_levels; i++)
-		if (axmap->levels[i].map)
-			free(axmap->levels[i].map);
-
-	free(axmap->levels);
-	free(axmap);
-	return NULL;
-}
-
-static bool axmap_handler(struct axmap *axmap, uint64_t bit_nr,
-			  bool (*func)(struct axmap_level *, unsigned long, unsigned int,
-			  void *), void *data)
-{
-	struct axmap_level *al;
-	int i;
-
-	for (i = 0; i < axmap->nr_levels; i++) {
-		unsigned long index = ulog64(bit_nr, i);
-		unsigned long offset = index >> UNIT_SHIFT;
-		unsigned int bit = index & BLOCKS_PER_UNIT_MASK;
-
-		al = &axmap->levels[i];
-
-		if (func(al, offset, bit, data))
-			return true;
-	}
-
-	return false;
-}
-
-static bool axmap_handler_topdown(struct axmap *axmap, uint64_t bit_nr,
-	bool (*func)(struct axmap_level *, unsigned long, unsigned int, void *),
-	void *data)
-{
-	struct axmap_level *al;
-	int i, level = axmap->nr_levels;
-
-	for (i = axmap->nr_levels - 1; i >= 0; i--) {
-		unsigned long index = ulog64(bit_nr, --level);
-		unsigned long offset = index >> UNIT_SHIFT;
-		unsigned int bit = index & BLOCKS_PER_UNIT_MASK;
-
-		al = &axmap->levels[i];
-
-		if (func(al, offset, bit, data))
-			return true;
-	}
-
-	return false;
-}
-
-static bool axmap_clear_fn(struct axmap_level *al, unsigned long offset,
-			   unsigned int bit, void *unused)
-{
-	if (!(al->map[offset] & (1UL << bit)))
-		return true;
-
-	al->map[offset] &= ~(1UL << bit);
-	return false;
-}
-
-void axmap_clear(struct axmap *axmap, uint64_t bit_nr)
-{
-	axmap_handler(axmap, bit_nr, axmap_clear_fn, NULL);
-
-	if (bit_nr < axmap->first_free)
-		axmap->first_free = bit_nr;
-}
-
-struct axmap_set_data {
-	unsigned int nr_bits;
-	unsigned int set_bits;
-};
-
 static const unsigned long bit_masks[] = {
 	0x0000000000000000, 0x0000000000000001, 0x0000000000000003, 0x0000000000000007,
 	0x000000000000000f, 0x000000000000001f, 0x000000000000003f, 0x000000000000007f,
@@ -216,6 +57,168 @@ static const unsigned long bit_masks[] = {
 #endif
 };
 
+/**
+ * struct axmap_level - a bitmap used to implement struct axmap
+ * @level: Level index. Each map has at least one level with index zero. The
+ *	higher the level index, the fewer bits a struct axmap_level contains.
+ * @map_size: Number of elements of the @map array.
+ * @map: A bitmap with @map_size elements.
+ */
+struct axmap_level {
+	int level;
+	unsigned long map_size;
+	unsigned long *map;
+};
+
+/**
+ * struct axmap - a set that can store numbers 0 .. @nr_bits - 1
+ * @nr_level: Number of elements of the @levels array.
+ * @levels: struct axmap_level array in which lower levels contain more bits
+ *	than higher levels.
+ * @nr_bits: One more than the highest value stored in the set.
+ */
+struct axmap {
+	unsigned int nr_levels;
+	struct axmap_level *levels;
+	uint64_t nr_bits;
+};
+
+/* Remove all elements from the @axmap set */
+void axmap_reset(struct axmap *axmap)
+{
+	int i;
+
+	for (i = 0; i < axmap->nr_levels; i++) {
+		struct axmap_level *al = &axmap->levels[i];
+
+		memset(al->map, 0, al->map_size * sizeof(unsigned long));
+	}
+}
+
+void axmap_free(struct axmap *axmap)
+{
+	unsigned int i;
+
+	if (!axmap)
+		return;
+
+	for (i = 0; i < axmap->nr_levels; i++)
+		free(axmap->levels[i].map);
+
+	free(axmap->levels);
+	free(axmap);
+}
+
+/* Allocate memory for a set that can store the numbers 0 .. @nr_bits - 1. */
+struct axmap *axmap_new(unsigned long nr_bits)
+{
+	struct axmap *axmap;
+	unsigned int i, levels;
+
+	axmap = malloc(sizeof(*axmap));
+	if (!axmap)
+		return NULL;
+
+	levels = 1;
+	i = (nr_bits + BLOCKS_PER_UNIT - 1) >> UNIT_SHIFT;
+	while (i > 1) {
+		i = (i + BLOCKS_PER_UNIT - 1) >> UNIT_SHIFT;
+		levels++;
+	}
+
+	axmap->nr_levels = levels;
+	axmap->levels = calloc(axmap->nr_levels, sizeof(struct axmap_level));
+	if (!axmap->levels)
+		goto free_axmap;
+	axmap->nr_bits = nr_bits;
+
+	for (i = 0; i < axmap->nr_levels; i++) {
+		struct axmap_level *al = &axmap->levels[i];
+
+		al->level = i;
+		al->map_size = (nr_bits + BLOCKS_PER_UNIT - 1) >> UNIT_SHIFT;
+		al->map = malloc(al->map_size * sizeof(unsigned long));
+		if (!al->map)
+			goto free_levels;
+
+		nr_bits = (nr_bits + BLOCKS_PER_UNIT - 1) >> UNIT_SHIFT;
+	}
+
+	axmap_reset(axmap);
+	return axmap;
+
+free_levels:
+	for (i = 0; i < axmap->nr_levels; i++)
+		free(axmap->levels[i].map);
+
+	free(axmap->levels);
+
+free_axmap:
+	free(axmap);
+	return NULL;
+}
+
+/*
+ * Call @func for each level, starting at level zero, until a level is found
+ * for which @func returns true. Return false if none of the @func calls
+ * returns true.
+ */
+static bool axmap_handler(struct axmap *axmap, uint64_t bit_nr,
+			  bool (*func)(struct axmap_level *, unsigned long, unsigned int,
+			  void *), void *data)
+{
+	struct axmap_level *al;
+	uint64_t index = bit_nr;
+	int i;
+
+	for (i = 0; i < axmap->nr_levels; i++) {
+		unsigned long offset = index >> UNIT_SHIFT;
+		unsigned int bit = index & BLOCKS_PER_UNIT_MASK;
+
+		al = &axmap->levels[i];
+
+		if (func(al, offset, bit, data))
+			return true;
+
+		if (index)
+			index >>= UNIT_SHIFT;
+	}
+
+	return false;
+}
+
+/*
+ * Call @func for each level, starting at the highest level, until a level is
+ * found for which @func returns true. Return false if none of the @func calls
+ * returns true.
+ */
+static bool axmap_handler_topdown(struct axmap *axmap, uint64_t bit_nr,
+	bool (*func)(struct axmap_level *, unsigned long, unsigned int, void *))
+{
+	int i;
+
+	for (i = axmap->nr_levels - 1; i >= 0; i--) {
+		unsigned long index = bit_nr >> (UNIT_SHIFT * i);
+		unsigned long offset = index >> UNIT_SHIFT;
+		unsigned int bit = index & BLOCKS_PER_UNIT_MASK;
+
+		if (func(&axmap->levels[i], offset, bit, NULL))
+			return true;
+	}
+
+	return false;
+}
+
+struct axmap_set_data {
+	unsigned int nr_bits;
+	unsigned int set_bits;
+};
+
+/*
+ * Set at most @__data->nr_bits bits in @al at offset @offset. Do not exceed
+ * the boundary of the element at offset @offset. Return the number of bits
+ * that have been set in @__data->set_bits if @al->level == 0.
+ */
 static bool axmap_set_fn(struct axmap_level *al, unsigned long offset,
 			 unsigned int bit, void *__data)
 {
@@ -231,59 +234,52 @@ static bool axmap_set_fn(struct axmap_level *al, unsigned long offset,
 	 * Mask off any potential overlap, only sets contig regions
 	 */
 	overlap = al->map[offset] & mask;
-	if (overlap == mask)
+	if (overlap == mask) {
+		data->set_bits = 0;
 		return true;
+	}
 
-	while (overlap) {
-		unsigned long clear_mask = ~(1UL << ffz(~overlap));
-
-		mask &= clear_mask;
-		overlap &= clear_mask;
-		nr_bits--;
+	if (overlap) {
+		nr_bits = ffz(~overlap) - bit;
+		if (!nr_bits)
+			return true;
+		mask = bit_masks[nr_bits] << bit;
 	}
 
 	assert(mask);
 	assert(!(al->map[offset] & mask));
-		
 	al->map[offset] |= mask;
 
 	if (!al->level)
 		data->set_bits = nr_bits;
 
+	/* For the next level */
 	data->nr_bits = 1;
+
 	return al->map[offset] != -1UL;
 }
 
+/*
+ * Set up to @data->nr_bits starting from @bit_nr in @axmap. Start at
+ * @bit_nr. If that bit has not yet been set then set it and continue until
+ * either @data->nr_bits have been set or a 1 bit is found. Store the number
+ * of bits that have been set in @data->set_bits. It is guaranteed that all
+ * bits that have been requested to set fit in the same unsigned long word of
+ * level 0 of @axmap.
+ */
 static void __axmap_set(struct axmap *axmap, uint64_t bit_nr,
 			 struct axmap_set_data *data)
 {
-	unsigned int set_bits, nr_bits = data->nr_bits;
-
-	if (axmap->first_free >= bit_nr &&
-	    axmap->first_free < bit_nr + data->nr_bits)
-		axmap->first_free = -1ULL;
+	unsigned int nr_bits = data->nr_bits;
 
 	if (bit_nr > axmap->nr_bits)
 		return;
 	else if (bit_nr + nr_bits > axmap->nr_bits)
 		nr_bits = axmap->nr_bits - bit_nr;
 
-	set_bits = 0;
-	while (nr_bits) {
-		axmap_handler(axmap, bit_nr, axmap_set_fn, data);
-		set_bits += data->set_bits;
+	assert(nr_bits <= BLOCKS_PER_UNIT);
 
-		if (!data->set_bits ||
-		    data->set_bits != (BLOCKS_PER_UNIT - nr_bits))
-			break;
-
-		nr_bits -= data->set_bits;
-		bit_nr += data->set_bits;
-
-		data->nr_bits = nr_bits;
-	}
-
-	data->set_bits = set_bits;
+	axmap_handler(axmap, bit_nr, axmap_set_fn, data);
 }
 
 void axmap_set(struct axmap *axmap, uint64_t bit_nr)
@@ -293,6 +289,12 @@ void axmap_set(struct axmap *axmap, uint64_t bit_nr)
 	__axmap_set(axmap, bit_nr, &data);
 }
 
+/*
+ * Set up to @nr_bits starting from @bit in @axmap. Start at @bit. If that
+ * bit has not yet been set then set it and continue until either @nr_bits
+ * have been set or a 1 bit is found. Return the number of bits that have been
+ * set.
+ */
 unsigned int axmap_set_nr(struct axmap *axmap, uint64_t bit_nr,
 			  unsigned int nr_bits)
 {
@@ -303,7 +305,7 @@ unsigned int axmap_set_nr(struct axmap *axmap, uint64_t bit_nr,
 		unsigned int max_bits, this_set;
 
 		max_bits = BLOCKS_PER_UNIT - (bit_nr & BLOCKS_PER_UNIT_MASK);
-		if (max_bits < nr_bits)
+		if (nr_bits > max_bits)
 			data.nr_bits = max_bits;
 
 		this_set = data.nr_bits;
@@ -328,108 +330,124 @@ static bool axmap_isset_fn(struct axmap_level *al, unsigned long offset,
 bool axmap_isset(struct axmap *axmap, uint64_t bit_nr)
 {
 	if (bit_nr <= axmap->nr_bits)
-		return axmap_handler_topdown(axmap, bit_nr, axmap_isset_fn, NULL);
+		return axmap_handler_topdown(axmap, bit_nr, axmap_isset_fn);
 
 	return false;
 }
 
-static uint64_t axmap_find_first_free(struct axmap *axmap, unsigned int level,
-				       uint64_t index)
+/*
+ * Find the first free bit that is at least as large as bit_nr.  Return
+ * -1 if no free bit is found before the end of the map.
+ */
+static uint64_t axmap_find_first_free(struct axmap *axmap, uint64_t bit_nr)
 {
-	uint64_t ret = -1ULL;
-	unsigned long j;
 	int i;
+	unsigned long temp;
+	unsigned int bit;
+	uint64_t offset, base_index, index;
+	struct axmap_level *al;
 
-	/*
-	 * Start at the bottom, then converge towards first free bit at the top
-	 */
-	for (i = level; i >= 0; i--) {
-		struct axmap_level *al = &axmap->levels[i];
+	index = 0;
+	for (i = axmap->nr_levels - 1; i >= 0; i--) {
+		al = &axmap->levels[i];
 
+		/* Shift previously calculated index for next level */
+		index <<= UNIT_SHIFT;
+
 		/*
-		 * Clear 'ret', this is a bug condition.
+		 * Start from an index that's at least as large as the
+		 * originally passed in bit number.
 		 */
-		if (index >= al->map_size) {
-			ret = -1ULL;
-			break;
-		}
+		base_index = bit_nr >> (UNIT_SHIFT * i);
+		if (index < base_index)
+			index = base_index;
 
-		for (j = index; j < al->map_size; j++) {
-			if (al->map[j] == -1UL)
-				continue;
+		/* Get the offset and bit for this level */
+		offset = index >> UNIT_SHIFT;
+		bit = index & BLOCKS_PER_UNIT_MASK;
 
-			/*
-			 * First free bit here is our index into the first
-			 * free bit at the next higher level
-			 */
-			ret = index = (j << UNIT_SHIFT) + ffz(al->map[j]);
-			break;
-		}
-	}
+		/*
+		 * If the previous level had unused bits in its last
+		 * word, the offset could be bigger than the map at
+		 * this level. That means no free bits exist before the
+		 * end of the map, so return -1.
+		 */
+		if (offset >= al->map_size)
+			return -1ULL;
 
-	if (ret < axmap->nr_bits)
-		return ret;
+		/* Check the first word starting with the specific bit */
+		temp = ~bit_masks[bit] & ~al->map[offset];
+		if (temp)
+			goto found;
 
-	return (uint64_t) -1ULL;
-}
+		/*
+		 * No free bit in the first word, so iterate
+		 * looking for a word with one or more free bits.
+		 */
+		for (offset++; offset < al->map_size; offset++) {
+			temp = ~al->map[offset];
+			if (temp)
+				goto found;
+		}
 
-static uint64_t axmap_first_free(struct axmap *axmap)
-{
-	if (!firstfree_valid(axmap))
-		axmap->first_free = axmap_find_first_free(axmap, axmap->nr_levels - 1, 0);
+		/* Did not find a free bit */
+		return -1ULL;
 
-	return axmap->first_free;
-}
-
-struct axmap_next_free_data {
-	unsigned int level;
-	unsigned long offset;
-	uint64_t bit;
-};
+found:
+		/* Compute the index of the free bit just found */
+		index = (offset << UNIT_SHIFT) + ffz(~temp);
+	}
 
-static bool axmap_next_free_fn(struct axmap_level *al, unsigned long offset,
-			       unsigned int bit, void *__data)
-{
-	struct axmap_next_free_data *data = __data;
-	uint64_t mask = ~bit_masks[(data->bit + 1) & BLOCKS_PER_UNIT_MASK];
-
-	if (!(mask & ~al->map[offset]))
-		return false;
-
-	if (al->map[offset] != -1UL) {
-		data->level = al->level;
-		data->offset = offset;
-		return true;
-	}
+	/* If found an unused bit in the last word of level 0, return -1 */
+	if (index >= axmap->nr_bits)
+		return -1ULL;
 
-	data->bit = (data->bit + BLOCKS_PER_UNIT - 1) / BLOCKS_PER_UNIT;
-	return false;
+	return index;
 }
 
 /*
  * 'bit_nr' is already set. Find the next free bit after this one.
+ * Return -1 if no free bits found.
  */
 uint64_t axmap_next_free(struct axmap *axmap, uint64_t bit_nr)
 {
-	struct axmap_next_free_data data = { .level = -1U, .bit = bit_nr, };
 	uint64_t ret;
+	uint64_t next_bit = bit_nr + 1;
+	unsigned long temp;
+	uint64_t offset;
+	unsigned int bit;
 
-	if (firstfree_valid(axmap) && bit_nr < axmap->first_free)
-		return axmap->first_free;
+	if (bit_nr >= axmap->nr_bits)
+		return -1ULL;
 
-	if (!axmap_handler(axmap, bit_nr, axmap_next_free_fn, &data))
-		return axmap_first_free(axmap);
+	/* If at the end of the map, wrap-around */
+	if (next_bit == axmap->nr_bits)
+		next_bit = 0;
 
-	assert(data.level != -1U);
+	offset = next_bit >> UNIT_SHIFT;
+	bit = next_bit & BLOCKS_PER_UNIT_MASK;
 
 	/*
-	 * In the rare case that the map is unaligned, we might end up
-	 * finding an offset that's beyond the valid end. For that case,
-	 * find the first free one, the map is practically full.
+	 * As an optimization, do a quick check for a free bit
+	 * in the current word at level 0. If not found, do
+	 * a topdown search.
 	 */
-	ret = axmap_find_first_free(axmap, data.level, data.offset);
-	if (ret != -1ULL)
-		return ret;
+	temp = ~bit_masks[bit] & ~axmap->levels[0].map[offset];
+	if (temp) {
+		ret = (offset << UNIT_SHIFT) + ffz(~temp);
 
-	return axmap_first_free(axmap);
+		/* Might have found an unused bit at level 0 */
+		if (ret >= axmap->nr_bits)
+			ret = -1ULL;
+	} else
+		ret = axmap_find_first_free(axmap, next_bit);
+
+	/*
+	 * If there are no free bits starting at next_bit and going
+	 * to the end of the map, wrap around by searching again
+	 * starting at bit 0.
+	 */
+	if (ret == -1ULL && next_bit != 0)
+		ret = axmap_find_first_free(axmap, 0);
+	return ret;
 }
